"""Package version information.
"""

__title__ = "welkin"
__description__ = "Python Welkin Health API Wrapper."
__url__ = "https://github.com/Lightmatter/welkin-health"
__version__ = "0.0.6"
__author__ = "Sam Morgan"
__author_email__ = "sam@lightmatter.com"
__license__ = "GNU GPL 3.0"
__copyright__ = "2022 Lightmatter"
